<?php

/**
 * Mail class for FramePress.
 *
 * This class is responsable for sending mails from the framework, import customs path from path.php user file.
 *
 * Licensed under The GPL v2 License
 * Redistributions of files must retain the above copyright notice.
 *
 * @package	   core
 * @subpackage	core.mail
 * @since		 beta 3
 * @license	   GPL v2 License
 * @creator	   Ausberto Huanca Vila
 * 
 * IMPORTANT NOTE: class name will be rewrited as mail_[prefix] (see init.php file), to get unique class names between plugins.
 * 
 */
 
class PressbackupMail
{

	/**
	 * Recipients list for the headers, includes: Name <mail>
	 *
	 * @var array
	 * @access public
	 */
	public $to = array();
	
	/**
	 * Recipients mail list for the mail function
	 *
	 * @var array
	 * @access public
	 */
	public $to_mails = array();
	
	/**
	 * Carbon copy (Cc) list that includes name and mail
	 * 
	 * @var array
	 * @access public
	 */
	public $cc = array();

	/**
	 * Blind Carbon Copy list that includes name and mail
	 * 
	 * @var array
	 * @access public
	 */
	public $Bcc = array();

	/**
	 * Address to reply the message
	 *
	 * @var string
	 * @access public
	 */
	public $ReplyTo = '';

	/**
	 * Name and email address from where the mail was sent
	 *
	 * @var array
	 * @access public
	 */
	public $from = array();

	/**
	 * The email message subject
	 *
	 * @var string
	 * @access public
	 */
	public $subject = '';

	/**
	 * Max length for the lines of the message (Don't apply to html messages)
	 * 
	 * @var integer
	 * @access public
	 */
	public $lineLength = 0;

	/**
	 * Type of message to send (html or plain text)
	 *
	 * @var string
	 * @access public
	 */
	public $mailType = 'html';

	/**
	 * Attachment to send in the email message, the file's full path name
	 *
	 * @var string
	 * @access public
	 */
	public $attachment = '';

	/**
	 * The template name to use for the mail message.
	 * The template file should be ubicated at the "views/mail" folder.
	 *
	 * @var string
	 * @access public
	 */
	public $template = '';

	/**
	 * List of variables to pass to the view
	 * 
	 * @var array
	 * @access public
	 */
	public $vars = array();


	/**
	 * Local Reference to the framework core object
	 * 
	 * @var object
	 * @access private
	 */
	private $fp = null;

	/**
	 * Constructor.
	 *
	 * @param string $FramePress Core Class
	 * @access public
	 */
	public function __construct( &$FramePress ){
		$this->fp = $FramePress;
	}

	/**
	 * Adds the from name and mail address
	 *
	 * @param string $mail
	 * @param string $name
	 * @access public
	 * @return void
	 */
	public function from($mail, $name='')
	{
		$this->from['mail'] = $mail;
		$this->from['name'] = ($name != "") ? $name . ' ' : '';
	}

	/**
	 * Adds recipients for the message.
	 * The names and mail addresses are added in the array for the header
	 * Only the addresses are added to the second array, for the mail function.
	 *
	 * @param string $mail
	 * @param string $name
	 * @access public
	 * @return void
	 */
	public function to($mail, $name='')
	{
		//Format the name and address according to the mail headers format: Name <address>
		$this->to[] = (($name != '') ? $name . ' ' : '') . '<' . $mail . '>';
		$this->to_mails[] = $mail;
	}

	/**
	 * Adds the Cc's addresses and names
	 *
	 * @param string $mail
	 * @param string $name
	 * @access public
	 * @return void
	 */
	public function cc($mail, $name='')
	{
		//Format the name and address according to the mail headers format: Name <address>
		$this->cc[] = (($name != '') ? $name . ' ' : '') . '<' . $mail . '>';
	}

	/**
	 * Adds the Bcc's addresses and names
	 *
	 * @param string $mail
	 * @param string $name
	 * @access public
	 * @return void
	 */
	public function Bcc($mail, $name='')
	{
		//Format the name and address according to the mail headers format: Name <address>
		$this->Bcc[] = (($name != '') ? $name . ' ' : '') . '<' . $mail . '>';
	}

	/**
	 * Save a variable to pass it to the template.
	 * 
	 * @param string $varName
	 * @param mixed $value
	 * @access public
	 * @return void
	 */
	public function set($varName, $value)
	{
		$this->vars[$varName] = $value;
	}

	/**
	 * Adds the Reply-to parameter
	 *
	 * @param string $mail
	 * @access public
	 * @return void
	 */
	public function ReplyTo($mail)
	{
		$this->ReplyTo = $mail;
	}

	/**
	 * Adds the attachment file to send with the email
	 *
	 * @param string $file
	 * @access public
	 * @return void
	 */
	public function attachment($file)
	{
		$this->attachment = $file;
	}

	/**
	 * Draw the view with the template and the vars.
	 * The template path is defined by the path class.
	 *
	 * @access public
	 * @return string
	 */
	public function draw()
	{
		//Extract the vars from the array, setting the key as variable and the value as the value.
		extract($this->vars);

		//require the file or the default email view
		$x1x_file = $this->fp->path->Dir['MAIL_VIEW'] . '/' . $this->template . ".php";
		if(!file_exists($x1x_fil)) {
			$x1x_file =  $this->fp->Path->Dir['D_VIEW'] . DS . "default_mail.php";
		}

		//get file content
		ob_start();
			require_once($x1x_file);
			$content = ob_get_contents();
		ob_end_clean();

		return $content;
	}

	/**
	 * Sends the mail and prepare the headers according to the values set before.
	 *
	 * @access public
	 * @return boolean
	 */
	public function send()
	{
		$headers = '';
		$body = '';

		//Gets the template with the merged vars
		$msg = $this->draw();

		//Makes the wordwrap only for plain text
		if ($this->lineLength > 0 && $this->mailType != 'html') {
			$msg = wordwrap($msg, $this->lineLength);
		}

		//Start to build the headers
		if (count($this->to) > 0) {
			$headers.='To: ' . implode(', ', $this->to) . "\r\n";
		}
		if ($this->from != "") {
			$headers.='From: ' . $this->from['name'] . '<' . $this->from['mail'] . '>' . "\r\n";
		}
		$headers.='X-Mailer: PHP/' . phpversion() . "\r\n";
		if (count($this->cc) > 0) {
			$headers.='Cc: ' . implode(', ', $this->cc) . "\r\n";
		}
		if (count($this->Bcc) > 0) {
			$headers.='Bcc: ' . implode(', ', $this->Bcc) . "\r\n";
		}
		if ($this->ReplyTo != "") {
			$headers.='Reply-To: ' . $this->ReplyTo . "\r\n";
		}

		//Make an unique id for the boundaries
		$uid = md5(uniqid(time()));
		$headers .= "MIME-Version: 1.0\r\n";

		//Special headers for an attachment
		if ($this->attachment != "") {
			$headers .= "Content-Type: multipart/mixed; boundary=\"mixed-" . $uid . "\"\r\n\r\n";
			$body .= "This is a multi-part message in MIME format.\r\n";
			$body .= "--mixed-" . $uid . "\r\n";
			$body .= "Content-Type: multipart/alternative;  boundary=\"alt-" . $uid . "\"\r\n";
			$body .= "--alt-" . $uid . "\r\n";
		}

		//Type of content for the mail message and its body
		if ($this->mailType == 'html') {
			if ($this->attachment != "") {
				$body .= 'Content-type: text/html; charset=iso-8859-1' . "\r\nContent-Transfer-Encoding: 7bit\r\n\r\n";
			} else {
				$headers .= 'Content-type: text/html; charset=iso-8859-1' . "\r\nContent-Transfer-Encoding: 7bit\r\n\r\n";
			}
		} else {
			if ($this->attachment != "") {
				$body .= 'Content-type: text/plain; charset=iso-8859-1' . "\r\nContent-Transfer-Encoding: 7bit\r\n\r\n";
			}else{
				$headers .= 'Content-type: text/plain; charset=iso-8859-1' . "\r\nContent-Transfer-Encoding: 7bit\r\n\r\n";
			}
		}
		$body .= $msg . "\r\n\r\n";

		//Attachment headers and content
		if ($this->attachment != "") {
			//Gets the file content to attach to the mail
			$handle = fopen($this->attachment, "rb");
			$content = fread($handle, filesize($this->attachment));
			fclose($handle);
			$content = chunk_split(base64_encode($content));

			//Prepare the headers for the file
			$body .= "--alt-" . $uid . "--\r\n\r\n";
			$body .= "--mixed-" . $uid . "\r\n";
			$body .= "Content-Type: application/octet-stream; name=\"" . basename($this->attachment) . "\"\r\n";
			$body .= "Content-Description: " . basename($this->attachment) . "\r\n";
			$body .= "Content-Disposition: attachment; filename=\"" . basename($this->attachment) . "\"; size=\"" . filesize($this->attachment) . "\";\r\n";
			$body .= "Content-Transfer-Encoding: base64\r\n\r\n";
			$body .= $content . "\r\n";
			$body .= "--mixed-" . $uid . "--";
		}
		return @mail(implode(', ', $this->to_mails), $this->subject, $body, $headers);
	}

}
?>
